"""Switcher water heater unofficial API and bridge, Device classes.

.. codeauthor:: Tomer Figenblat <tomer.figenblat@gmail.com>

"""

from datetime import datetime
from typing import Optional


class SwitcherV2Device:
    """Represntation of the switcherv2 device.

    Args:
      device_id: the id retrieved from the device.
      ip_address: the ip address assigned to the device.
      mac_address: the mac address assigned to the device.
      name: the name of the device.
      state: the current state of the device (in/off).
      remaining_time: remaining time (if on).
      auto_off_set: configured value for auto shutdown.
      power_consumption: the current power consumpstion in watts.
      electric_current: the current power consumpstion in amps.
      phone_id: the phone id retrieved from the device.
      device_password: the password retrieved from the device.
      last_state_change: datetime of the last state change.

    """

    def __init__(
        self,
        device_id: str,
        ip_address: str,
        mac_address: str,
        name: str,
        state: str,
        remaining_time: Optional[str],
        auto_off_set: str,
        power_consumption: int,
        electric_current: float,
        phone_id: str,
        device_password: str,
        last_state_change: datetime,
    ) -> None:
        """Initialize the Switcher V2 Device."""
        self._device_id = device_id
        self._mac_addr = mac_address
        self._phone_id = phone_id
        self._device_password = device_password
        self.update_device_data(
            ip_address,
            name,
            state,
            remaining_time,
            auto_off_set,
            power_consumption,
            electric_current,
            last_state_change,
        )

    def update_device_data(
        self,
        ip_address: str,
        name: str,
        state: str,
        remaining_time: Optional[str],
        auto_off_set: str,
        power_consumption: int,
        electric_current: float,
        last_state_change: datetime,
    ) -> None:
        """Update the device state and data.

        Args:
          ip_address: the ip address assigned to the device.
          name: the name of the device.
          state: the current state of the device (on/off).
          remaining_time: remaining time (if on).
          auto_off_set: configured value for auto shutdown.
          power_consumption: the current power consumpstion in watts.
          electric_current: the current power consumpstion in amps.
          last_state_change: datetime of the last state change.

        """
        self._ip_address = ip_address
        self._name = name
        self._state = state
        self._remaining_time = remaining_time
        self._auto_off_set = auto_off_set
        self._power_consumption = power_consumption
        self._electric_current = electric_current
        self._last_data_update = datetime.now()
        self._last_state_change = last_state_change

    @property
    def device_id(self) -> str:
        """str: Returns the device id."""
        return self._device_id

    @property
    def ip_addr(self) -> str:
        """str: Returns the ip address."""
        return self._ip_address

    @property
    def mac_addr(self) -> str:
        """str: Returns the mac address."""
        return self._mac_addr

    @property
    def name(self) -> str:
        """str: Returns the device name."""
        return self._name

    @property
    def state(self) -> str:
        """str: Returns the device state."""
        return self._state

    @property
    def remaining_time(self) -> Optional[str]:
        """str: Returns the time left to auto shutdown."""
        return self._remaining_time

    @property
    def auto_off_set(self) -> str:
        """str: Returns the auto-off configuration value."""
        return self._auto_off_set

    @property
    def power_consumption(self) -> int:
        """int: Returns the power consumption in watts."""
        return self._power_consumption

    @property
    def electric_current(self) -> float:
        """float: returns the power consumption in amps."""
        return self._electric_current

    @property
    def phone_id(self) -> str:
        """str: Returns the phone id."""
        return self._phone_id

    @property
    def device_password(self) -> str:
        """str: Returns the device password."""
        return self._device_password

    @property
    def last_data_update(self) -> datetime:
        """datetime: Returns timestamp of the last update."""
        return self._last_data_update

    @property
    def last_state_change(self) -> datetime:
        """datetime: Returns timestamp of the last state change."""
        return self._last_state_change

    def as_dict(self):
        """Return as dict.

        Returns:
          A dictionary represntation of the object properties.
          Used to make the object json serializable.


        """
        return self.__dict__



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['aioswitcher', 'aioswitcher.api', 'aioswitcher.bridge']

package_data = \
{'': ['*']}

extras_require = \
{'checkers': ['bandit==1.6.2',
              'black==20.8b1',
              'doc8==0.8.1',
              'flake8==3.8.3',
              'flake8-docstrings==1.5.0',
              'isort==5.4.2',
              'mypy==0.782',
              'pygments==2.6.1',
              'yamllint==1.24.2'],
 'tests': ['asynctest==0.13.0',
           'codecov==2.1.9',
           'pytest==6.0.1',
           'pytest-aiohttp==0.3.0',
           'pytest-asyncio==0.14.0',
           'pytest-cov==2.10.1']}

setup_kwargs = {
    'name': 'aioswitcher',
    'version': '1.2.1',
    'description': 'Switcher Water Heater Unofficial Bridge and API.',
    'long_description': "# Switcher Water Heater Unofficial Bridge and API</br>[![pypi-version]][11] [![pypi-downloads]][11] [![license-badge]][4] [![conventional-commits]][0]\n\n[![gh-build-status]][7] [![read-the-docs]][8] [![codecov]][3] [![fossa-status]][5]\n\nPyPi module named [aioswitcher][11] for integrating with the [Switcher Water Heater](https://www.switcher.co.il/).</br>\nPlease check out the [documentation][8].\n\n## Install\n\n```shell\npip install aioswitcher\n```\n\n## Usage Example\n\nPlease check out the [documentation][8] for the full usage section.\n\n```python\nasync with SwitcherV2Api(\n        your_loop, ip_address, phone_id,\n        device_id, device_password) as swapi:\n    # get the device state\n    state_response = await swapi.get_state()\n\n    # control the device: on / off / on + 30 minutes timer\n    turn_on_response = await swapi.control_device(consts.COMMAND_ON)\n    turn_off_response = await swapi.control_device(consts.COMMAND_OFF)\n    turn_on_30_min_response = await swapi.control_device(consts.COMMAND_ON, '30')\n```\n\n## Contributing\n\nThe contributing guidelines are [here](.github/CONTRIBUTING.md)\n\n## Code of Conduct\n\nThe code of conduct is [here](.github/CODE_OF_CONDUCT.md)\n\n<!-- Real Links -->\n[0]: https://conventionalcommits.org\n[2]: https://github.com/TomerFi/aioswitcher/releases\n[3]: https://codecov.io/gh/TomerFi/aioswitcher\n[4]: https://github.com/TomerFi/aioswitcher\n[5]: https://app.fossa.com/projects/git%2Bgithub.com%2FTomerFi%2Faioswitcher?ref=badge_shield\n[7]: https://github.com/TomerFi/aioswitcher/actions?query=workflow%3ABuild\n[8]: https://aioswitcher.tomfi.info/\n[11]: https://pypi.org/project/aioswitcher\n<!-- Badges Links -->\n[codecov]: https://codecov.io/gh/TomerFi/aioswitcher/graph/badge.svg\n[conventional-commits]: https://img.shields.io/badge/Conventional%20Commits-1.0.0-yellow.svg\n[fossa-status]: https://app.fossa.com/api/projects/git%2Bgithub.com%2FTomerFi%2Faioswitcher.svg?type=shield\n[gh-build-status]: https://github.com/TomerFi/aioswitcher/workflows/Build/badge.svg\n[license-badge]: https://img.shields.io/github/license/tomerfi/aioswitcher\n[pypi-downloads]: https://img.shields.io/pypi/dm/aioswitcher.svg?logo=pypi&color=1082C2\n[pypi-version]: https://img.shields.io/pypi/v/aioswitcher?logo=pypi\n[read-the-docs]: https://readthedocs.org/projects/aioswitcher/badge/?version=stable\n",
    'author': 'Tomer Figenblat',
    'author_email': 'tomer.figenblat@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/aioswitcher/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""Switcher Bridge Response Messages."""

# fmt: off
from asyncio import AbstractEventLoop, Future, ensure_future
from binascii import hexlify
from socket import inet_ntoa
from struct import pack
from typing import Optional, Union

from ..consts import (ENCODING_CODEC, STATE_OFF, STATE_ON, STATE_RESPONSE_ON,
                      WAITING_TEXT)
from ..tools import convert_seconds_to_iso_time

# fmt: on


class SwitcherV2BroadcastMSG:
    """Represntation of the SwitcherV2 broadcast message.

    Args:
      loop: the event loop to perform actions in.
      message: the raw message from the device.

    Todo:
      * replace ``init_future`` attribute with ``get_init_future`` method.

    """

    def __init__(
        self, loop: AbstractEventLoop, message: Union[bytes, str]
    ) -> None:
        """Initialize the broadcast message."""
        self._loop = loop
        self._verified = self._validated = False
        self._power_consumption = 0
        self._electric_current = 0.0
        self._ip_address = WAITING_TEXT
        self._mac_address = WAITING_TEXT
        self._name = WAITING_TEXT
        self._device_id = WAITING_TEXT
        self._auto_off_set = WAITING_TEXT
        self._remaining_time_to_off = WAITING_TEXT
        self._init_future = loop.create_future()
        fixed_msg = (
            message
            if isinstance(message, bytes)
            else message.encode(ENCODING_CODEC)
        )
        ensure_future(self.initialize(fixed_msg), loop=loop)

    async def initialize(self, message: bytes) -> None:
        """Finish the initialization and update the future object.

        Args:
          message: the raw message from the device.

        """
        try:
            self._verified = (
                hexlify(message)[0:4].decode(ENCODING_CODEC) == "fef0"
                and len(message) == 165
            )
            if self._verified:
                temp_ip = hexlify(message)[152:160]
                ip_addr = int(
                    temp_ip[6:8] + temp_ip[4:6] + temp_ip[2:4] + temp_ip[0:2],
                    16,
                )
                self._ip_address = inet_ntoa(pack("<L", ip_addr))

                mac = hexlify(message)[160:172].decode(ENCODING_CODEC).upper()
                self._mac_address = (
                    mac[0:2]
                    + ":"
                    + mac[2:4]
                    + ":"
                    + mac[4:6]
                    + ":"
                    + mac[6:8]
                    + ":"
                    + mac[8:10]
                    + ":"
                    + mac[10:12]
                )

                self._name = (
                    message[42:74].decode(ENCODING_CODEC).rstrip("\x00")
                )

                self._device_id = hexlify(message)[36:42].decode(
                    ENCODING_CODEC
                )

                self._device_state = (
                    STATE_ON
                    if hexlify(message)[266:270].decode(ENCODING_CODEC)
                    == STATE_RESPONSE_ON
                    else STATE_OFF
                )

                temp_auto_off_set = hexlify(message)[310:318]
                temp_auto_off_set_seconds = int(
                    temp_auto_off_set[6:8]
                    + temp_auto_off_set[4:6]
                    + temp_auto_off_set[2:4]
                    + temp_auto_off_set[0:2],
                    16,
                )
                self._auto_off_set = await convert_seconds_to_iso_time(
                    self._loop, temp_auto_off_set_seconds
                )

                if self._device_state == STATE_ON:
                    temp_power = hexlify(message)[270:278]
                    self._power_consumption = int(
                        temp_power[2:4] + temp_power[0:2], 16
                    )
                    self._electric_current = round(
                        (self._power_consumption / float(220)), 1
                    )

                    temp_remaining_time = hexlify(message)[294:302]
                    temp_remaining_time_seconds = int(
                        temp_remaining_time[6:8]
                        + temp_remaining_time[4:6]
                        + temp_remaining_time[2:4]
                        + temp_remaining_time[0:2],
                        16,
                    )
                    self._remaining_time_to_off = (
                        await convert_seconds_to_iso_time(
                            self._loop, temp_remaining_time_seconds
                        )
                    )

            self._validated = True
            self.init_future.set_result(self)
        except (ValueError, IndexError, RuntimeError) as ex:
            self.init_future.set_exception(ex)
        return None

    @property
    def verified(self) -> bool:
        """bool: Return whether or not the message is a SwitcherV2 message."""
        return self._verified if self._validated else self._validated

    @property
    def ip_address(self) -> str:
        """str: Return the ip address."""
        return self._ip_address

    @property
    def mac_address(self) -> str:
        """str: Return the mac address."""
        return self._mac_address

    @property
    def name(self) -> str:
        """str: Return the device name."""
        return self._name

    @property
    def device_id(self) -> str:
        """str: Return the device id."""
        return self._device_id

    @property
    def power(self) -> int:
        """int: Return the power consumptionin watts."""
        return self._power_consumption

    @property
    def device_state(self) -> str:
        """str: Return the state of the device."""
        return self._device_state

    @property
    def remaining_time_to_off(self) -> Optional[str]:
        """str: Return the time left to auto-off."""
        return self._remaining_time_to_off

    @property
    def current(self) -> float:
        """float: Return the power consumptionin amps."""
        return self._electric_current

    @property
    def auto_off_set(self) -> str:
        """str: Return the auto-off configuration value."""
        return self._auto_off_set

    @property
    def init_future(self) -> Future:
        """asyncio.Future: Return the future of the device initialization."""
        return self._init_future



"""Switcher water heater unofficial API and bridge, Exception classes.

.. codeauthor:: Tomer Figenblat <tomer.figenblat@gmail.com>

"""


class CalculationError(Exception):
    """Exception to be raised when cpu bound calculation is failing."""


class DecodingError(Exception):
    """Exception to be raised when cpu bound decoding is failing."""


class EncodingError(Exception):
    """Exception to be raised when cpu bound encoding is failing."""
